@@ -12,6 +12,8 @@ from __future__ import absolute_import, unicode_literals
 import logging
 import re
 
+import salt.utils.platform
+
 # Import Third Party Libs
 try:
     import psutil
@@ -83,24 +85,38 @@ def beacon(config):
     it will override the previously defined threshold.
 
     '''
-    parts = psutil.disk_partitions(all=False)
+    parts = psutil.disk_partitions(all=True)
     ret = []
     for mounts in config:
         mount = next(iter(mounts))
 
+        # Because we're using regular expressions
+        # if our mount doesn't end with a $, insert one.
+        mount_re = mount
+        if not mount.endswith('$'):
+            mount_re = '{0}$'.format(mount)
+
+        if salt.utils.platform.is_windows():
+            # mount_re comes in formatted with a $ at the end
+            # can be `C:\\$` or `C:\\\\$`
+            # re string must be like `C:\\\\` regardless of \\ or \\\\
+            # also, psutil returns uppercase
+            mount_re = re.sub(r':\\\$', r':\\\\', mount_re)
+            mount_re = re.sub(r':\\\\\$', r':\\\\', mount_re)
+            mount_re = mount_re.upper()
+
         for part in parts:
-            if re.match(mount, part.mountpoint):
+            if re.match(mount_re, part.mountpoint):
                 _mount = part.mountpoint
 
                 try:
-                    _current_usage = psutil.disk_usage(mount)
+                    _current_usage = psutil.disk_usage(_mount)
                 except OSError:
-                    log.warning('%s is not a valid mount point.', mount)
+                    log.warning('%s is not a valid mount point.', _mount)
                     continue
 
                 current_usage = _current_usage.percent
                 monitor_usage = mounts[mount]
-                log.info('current_usage %s', current_usage)
                 if '%' in monitor_usage:
                     monitor_usage = re.sub('%', '', monitor_usage)
                 monitor_usage = float(monitor_usage)
