@@ -11,7 +11,7 @@ Watch files and translate the changes into salt events
             the beacon configuration.
 
 :note: The `inotify` beacon only works on OSes that have `inotify`
-       kernel support.  Currently this excludes FreeBSD, macOS, and Windows.
+       kernel support.
 
 '''
 # Import Python libs
@@ -238,21 +238,21 @@ def beacon(config):
                     break
                 path = os.path.dirname(path)
 
-            for path in _config.get('files', {}):
-                excludes = _config['files'][path].get('exclude', '')
+            excludes = _config['files'][path].get('exclude', '')
 
             if excludes and isinstance(excludes, list):
                 for exclude in excludes:
                     if isinstance(exclude, dict):
-                        if exclude.values()[0].get('regex', False):
+                        _exclude = next(iter(exclude))
+                        if exclude[_exclude].get('regex', False):
                             try:
-                                if re.search(list(exclude)[0], event.pathname):
+                                if re.search(_exclude, event.pathname):
                                     _append = False
                             except Exception:
                                 log.warning('Failed to compile regex: %s',
-                                            list(exclude)[0])
+                                            _exclude)
                         else:
-                            exclude = list(exclude)[0]
+                            exclude = _exclude
                     elif '*' in exclude:
                         if fnmatch.fnmatch(event.pathname, exclude):
                             _append = False
@@ -312,7 +312,7 @@ def beacon(config):
                 excl = []
                 for exclude in excludes:
                     if isinstance(exclude, dict):
-                        excl.append(exclude.keys()[0])
+                        excl.append(list(exclude)[0])
                     else:
                         excl.append(exclude)
                 excl = pyinotify.ExcludeFilter(excl)
