@@ -5,17 +5,19 @@ Generate chronos proxy minion grains.
 .. versionadded:: 2015.8.2
 
 '''
-from __future__ import absolute_import
+# Import Python libs
+from __future__ import absolute_import, print_function, unicode_literals
 
 
-import salt.utils
+# Import Salt libs
 import salt.utils.http
+import salt.utils.platform
 __proxyenabled__ = ['chronos']
 __virtualname__ = 'chronos'
 
 
 def __virtual__():
-    if not salt.utils.is_proxy() or 'proxy' not in __opts__:
+    if not salt.utils.platform.is_proxy() or 'proxy' not in __opts__:
         return False
     else:
         return __virtualname__
