@@ -4,14 +4,18 @@ Support for Alternatives system
 
 :codeauthor: Radek Rada <radek.rada@gmail.com>
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import os
 import logging
 
 # Import Salt libs
-import salt.utils
+import salt.utils.files
+import salt.utils.path
+
+# Import 3rd-party libs
+from salt.ext import six
 
 
 __outputter__ = {
@@ -86,16 +90,15 @@ def show_link(name):
     path += 'alternatives/{0}'.format(name)
 
     try:
-        with salt.utils.fopen(path, 'rb') as r_file:
-            return r_file.readlines()[1].rstrip('\n')
+        with salt.utils.files.fopen(path, 'rb') as r_file:
+            contents = salt.utils.stringutils.to_unicode(r_file.read())
+            return contents.splitlines(True)[1].rstrip('\n')
     except OSError:
-        log.error(
-            'alternatives: {0} does not exist'.format(name)
-        )
+        log.error('alternatives: %s does not exist', name)
     except (IOError, IndexError) as exc:
         log.error(
-            'alternatives: unable to get master link for {0}. '
-            'Exception: {1}'.format(name, exc)
+            'alternatives: unable to get master link for %s. '
+            'Exception: %s', name, exc
         )
 
     return False
@@ -112,13 +115,10 @@ def show_current(name):
 
         salt '*' alternatives.show_current editor
     '''
-    alt_link_path = '/etc/alternatives/{0}'.format(name)
     try:
-        return os.readlink(alt_link_path)
+        return _read_link(name)
     except OSError:
-        log.error(
-            'alternatives: path {0} does not exist'.format(alt_link_path)
-        )
+        log.error('alternative: %s does not exist', name)
     return False
 
 
@@ -154,7 +154,10 @@ def check_installed(name, path):
 
         salt '*' alternatives.check_installed name path
     '''
-    return show_current(name) == path
+    try:
+        return _read_link(name) == path
+    except OSError:
+        return False
 
 
 def install(name, link, path, priority):
@@ -167,7 +170,7 @@ def install(name, link, path, priority):
 
         salt '*' alternatives.install editor /usr/bin/editor /usr/bin/emacs23 50
     '''
-    cmd = [_get_cmd(), '--install', link, name, path, str(priority)]
+    cmd = [_get_cmd(), '--install', link, name, path, six.text_type(priority)]
     out = __salt__['cmd.run_all'](cmd, python_shell=False)
     if out['retcode'] > 0 and out['stderr'] != '':
         return out['stderr']
@@ -224,3 +227,13 @@ def set_(name, path):
     if out['retcode'] > 0:
         return out['stderr']
     return out['stdout']
+
+
+def _read_link(name):
+    '''
+    Read the link from /etc/alternatives
+
+    Throws an OSError if the link does not exist
+    '''
+    alt_link_path = '/etc/alternatives/{0}'.format(name)
+    return salt.utils.path.readlink(alt_link_path)
