@@ -36,7 +36,7 @@ Module for handling openstack glance calls.
 '''
 
 # Import Python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 import re
 
 # Import salt libs
@@ -48,6 +48,9 @@ from salt.version import (
     __version__,
     SaltStackVersion
     )
+
+from salt.ext import six
+
 # is there not SaltStackVersion.current() to get
 # the version of the salt running this code??
 _version_ary = __version__.split('.')
@@ -100,6 +103,13 @@ def _auth(profile=None, api_version=2, **connection_args):
 
     Only intended to be used within glance-enabled modules
     '''
+    __utils__['versions.warn_until'](
+        'Neon',
+        (
+            'The glance module has been deprecated and will be removed in {version}.  '
+            'Please update to using the glanceng module'
+        ),
+    )
 
     if profile:
         prefix = profile + ":keystone."
@@ -309,10 +319,10 @@ def image_delete(id=None, name=None, profile=None):  # pylint: disable=C0103
             'comment': 'No image with ID {0}'.format(id)
             }
     except exc.HTTPForbidden as forbidden:
-        log.error(str(forbidden))
+        log.error(six.text_type(forbidden))
         return {
             'result': False,
-            'comment': str(forbidden)
+            'comment': six.text_type(forbidden)
             }
     return {
         'result': True,
