@@ -97,7 +97,8 @@ def diff_objects(obja, objb):
     Diff two complex infoblox objects.
     This is used from salt states to detect changes in objects.
 
-    Using `func:nextavailableip` will not cause a diff if the ipaddres is in range
+    Using ``func:nextavailableip`` will not cause a diff if the ipaddress is in
+    range
     '''
     return libinfoblox.diff_obj(obja, objb)
 
@@ -108,6 +109,8 @@ def is_ipaddr_in_ipfunc_range(ipaddr, ipfunc):
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.is_ipaddr_in_ipfunc_range \
             ipaddr="10.0.2.2" ipfunc="func:nextavailableip:10.0.0.0/8"
     '''
@@ -118,10 +121,12 @@ def update_host(name, data, **api_opts):
     '''
     Update host record. This is a helper call to update_object.
 
-    Find a hosts `_ref` then call update_object with the record data.
+    Find a hosts ``_ref`` then call update_object with the record data.
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.update_host name=fqdn data={}
     '''
     o = get_host(name=name, **api_opts)
@@ -130,8 +135,7 @@ def update_host(name, data, **api_opts):
 
 def update_object(objref, data, **api_opts):
     '''
-    Update raw infoblox object.
-    This is a low level api call.
+    Update raw infoblox object. This is a low level api call.
 
     CLI Example:
 
@@ -147,11 +151,12 @@ def update_object(objref, data, **api_opts):
 
 def delete_object(objref, **api_opts):
     '''
-    Delete infoblox object.
-    This is a low level api call.
+    Delete infoblox object. This is a low level api call.
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.delete_object objref=[ref_of_object]
     '''
     if '__opts__' in globals() and __opts__['test']:
@@ -162,11 +167,12 @@ def delete_object(objref, **api_opts):
 
 def create_object(object_type, data, **api_opts):
     '''
-    Create raw infoblox object
-    This is a low level api call.
+    Create raw infoblox object. This is a low level api call.
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.update_object object_type=record:host  data={}
     '''
     if '__opts__' in globals() and __opts__['test']:
@@ -178,11 +184,12 @@ def create_object(object_type, data, **api_opts):
 def get_object(objref, data=None, return_fields=None, max_results=None,
                ensure_none_or_one_result=False, **api_opts):
     '''
-    Get raw infoblox object.
-    This is a low level api call.
+    Get raw infoblox object. This is a low level api call.
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.get_object objref=[_ref of object]
     '''
     if not data:
@@ -198,6 +205,8 @@ def create_cname(data, **api_opts):
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.create_cname data={ \
             "comment": "cname to example server", \
             "name": "example.example.com", \
@@ -215,7 +224,9 @@ def get_cname(name=None, canonical=None, return_fields=None, **api_opts):
     '''
     Get CNAME information.
 
-    CLI Example:
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt-call infoblox.get_cname name=example.example.com
         salt-call infoblox.get_cname canonical=example-ha-0.example.com
@@ -229,10 +240,12 @@ def update_cname(name, data, **api_opts):
     '''
     Update CNAME. This is a helper call to update_object.
 
-    Find a CNAME `_ref` then call update_object with the record data.
+    Find a CNAME ``_ref`` then call update_object with the record data.
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.update_cname name=example.example.com data="{
                 'canonical':'example-ha-0.example.com',
                 'use_ttl':true,
@@ -251,7 +264,9 @@ def delete_cname(name=None, canonical=None, **api_opts):
 
     If record is not found, return True
 
-    CLI Example:
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt-call infoblox.delete_cname name=example.example.com
         salt-call infoblox.delete_cname canonical=example-ha-0.example.com
@@ -266,16 +281,13 @@ def get_host(name=None, ipv4addr=None, mac=None, return_fields=None, **api_opts)
     '''
     Get host information
 
-    CLI Example:
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt-call infoblox.get_host hostname.domain.ca
         salt-call infoblox.get_host ipv4addr=123.123.122.12
         salt-call infoblox.get_host mac=00:50:56:84:6e:ae
-
-        return_fields=
-        https://INFOBLOX/wapidoc/objects/record.host.html#fields-list
-
-        return_fields='ipv4addrs,aliases,name,configure_for_dns,extattrs,disable,view,comment,zone'
     '''
     infoblox = _get_infoblox(**api_opts)
     host = infoblox.get_host(name=name, mac=mac, ipv4addr=ipv4addr, return_fields=return_fields)
@@ -288,6 +300,8 @@ def get_host_advanced(name=None, ipv4addr=None, mac=None, **api_opts):
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.get_host_advanced hostname.domain.ca
     '''
     infoblox = _get_infoblox(**api_opts)
@@ -299,7 +313,8 @@ def get_host_domainname(name, domains=None, **api_opts):
     '''
     Get host domain name
 
-    If no domains are passed, the hostname is checked for a zone in infoblox, if no zone split on first dot.
+    If no domains are passed, the hostname is checked for a zone in infoblox,
+    if no zone split on first dot.
 
     If domains are provided, the best match out of the list is returned.
 
@@ -309,6 +324,8 @@ def get_host_domainname(name, domains=None, **api_opts):
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call uwl.get_host_domainname name=localhost.t.domain.com \
             domains=['domain.com', 't.domain.com.']
 
@@ -337,15 +354,19 @@ def get_host_hostname(name, domains=None, **api_opts):
     '''
     Get hostname
 
-    If no domains are passed, the hostname is checked for a zone in infoblox, if no zone split on first dot.
+    If no domains are passed, the hostname is checked for a zone in infoblox,
+    if no zone split on first dot.
 
-    If domains are provided, the best match out of the list is truncated from the fqdn leaving the hostname.
+    If domains are provided, the best match out of the list is truncated from
+    the fqdn leaving the hostname.
 
     If no matching domains are found the fqdn is returned.
 
     dots at end of names are ignored.
 
-    CLI Example:
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt-call infoblox.get_host_hostname fqdn=localhost.xxx.t.domain.com \
             domains="['domain.com', 't.domain.com']"
@@ -367,10 +388,12 @@ def get_host_mac(name=None, allow_array=False, **api_opts):
     '''
     Get mac address from host record.
 
-    Use `allow_array` to return possible mutiple values.
+    Use `allow_array` to return possible multiple values.
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.get_host_mac host=localhost.domain.com
     '''
     data = get_host(name=name, **api_opts)
@@ -390,9 +413,11 @@ def get_host_ipv4(name=None, mac=None, allow_array=False, **api_opts):
     '''
     Get ipv4 address from host record.
 
-    Use `allow_array` to return possible mutiple values.
+    Use `allow_array` to return possible multiple values.
 
-    CLI Example:
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt-call infoblox.get_host_ipv4 host=localhost.domain.com
         salt-call infoblox.get_host_ipv4 mac=00:50:56:84:6e:ae
@@ -416,14 +441,13 @@ def get_host_ipv4addr_info(ipv4addr=None, mac=None,
     '''
     Get host ipv4addr information
 
-    return_fields='mac,host,configure_for_dhcp,ipv4addr'
+    CLI Examples:
 
-    CLI Example:
+    .. code-block:: bash
 
         salt-call infoblox.get_ipv4addr ipv4addr=123.123.122.12
         salt-call infoblox.get_ipv4addr mac=00:50:56:84:6e:ae
-        salt-call infoblox.get_ipv4addr mac=00:50:56:84:6e:ae return_fields=host
-        return_fields='mac,host,configure_for_dhcp,ipv4addr'
+        salt-call infoblox.get_ipv4addr mac=00:50:56:84:6e:ae return_fields=host return_fields='mac,host,configure_for_dhcp,ipv4addr'
     '''
     infoblox = _get_infoblox(**api_opts)
     return infoblox.get_host_ipv4addr_object(ipv4addr, mac, discovered_data, return_fields)
@@ -437,6 +461,8 @@ def get_host_ipv6addr_info(ipv6addr=None, mac=None,
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.get_host_ipv6addr_info ipv6addr=2001:db8:85a3:8d3:1349:8a2e:370:7348
     '''
     infoblox = _get_infoblox(**api_opts)
@@ -445,9 +471,8 @@ def get_host_ipv6addr_info(ipv6addr=None, mac=None,
 
 def get_network(ipv4addr=None, network=None, return_fields=None, **api_opts):
     '''
-    Get list of all networks.
-    This is helpfull when looking up subnets to
-    use with func:nextavailableip
+    Get list of all networks. This is helpful when looking up subnets to use
+    with func:nextavailableip
 
     This call is offen slow and not cached!
 
@@ -456,6 +481,8 @@ def get_network(ipv4addr=None, network=None, return_fields=None, **api_opts):
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.get_network
     '''
     infoblox = _get_infoblox(**api_opts)
@@ -468,6 +495,8 @@ def delete_host(name=None, mac=None, ipv4addr=None, **api_opts):
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.delete_host name=example.domain.com
         salt-call infoblox.delete_host ipv4addr=123.123.122.12
         salt-call infoblox.delete_host ipv4addr=123.123.122.12 mac=00:50:56:84:6e:ae
@@ -483,15 +512,18 @@ def create_host(data, **api_opts):
     Add host record
 
     Avoid race conditions, use func:nextavailableip for ipv[4,6]addrs:
-        - func:nextavailableip:network/ZG54dfgsrDFEFfsfsLzA:10.0.0.0/8/default
-        - func:nextavailableip:10.0.0.0/8
-        - func:nextavailableip:10.0.0.0/8,external
-        - func:nextavailableip:10.0.0.3-10.0.0.10
+
+    - func:nextavailableip:network/ZG54dfgsrDFEFfsfsLzA:10.0.0.0/8/default
+    - func:nextavailableip:10.0.0.0/8
+    - func:nextavailableip:10.0.0.0/8,external
+    - func:nextavailableip:10.0.0.3-10.0.0.10
 
     See your infoblox API for full `data` format.
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.create_host \
             data =
                 {'name': 'hostname.example.ca',
@@ -514,6 +546,8 @@ def get_ipv4_range(start_addr=None, end_addr=None, return_fields=None, **api_opt
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.get_ipv4_range start_addr=123.123.122.12
     '''
     infoblox = _get_infoblox(**api_opts)
@@ -526,7 +560,9 @@ def delete_ipv4_range(start_addr=None, end_addr=None, **api_opts):
 
     CLI Example:
 
-            salt-call infoblox.delete_ipv4_range start_addr=123.123.122.12
+    .. code-block:: bash
+
+        salt-call infoblox.delete_ipv4_range start_addr=123.123.122.12
     '''
     r = get_ipv4_range(start_addr, end_addr, **api_opts)
     if r:
@@ -544,6 +580,8 @@ def create_ipv4_range(data, **api_opts):
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.create_ipv4_range data={
             start_addr: '129.97.150.160',
             end_addr: '129.97.150.170'}
@@ -560,6 +598,8 @@ def create_a(data, **api_opts):
 
     CLI Example:
 
+    .. code-block:: bash
+
         salt-call infoblox.create_a \
                     data =
                     name: 'fastlinux.math.example.ca'
@@ -573,7 +613,9 @@ def get_a(name=None, ipv4addr=None, allow_array=True, **api_opts):
     '''
     Get A record
 
-    CLI Example:
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt-call infoblox.get_a name=abc.example.com
         salt-call infoblox.get_a ipv4addr=192.168.3.5
@@ -593,14 +635,16 @@ def delete_a(name=None, ipv4addr=None, allow_array=False, **api_opts):
     '''
     Delete A record
 
-    If the A record is used as a round robin you can set
-    `allow_array=true to delete all records for the hostname.
+    If the A record is used as a round robin you can set ``allow_array=True`` to
+    delete all records for the hostname.
 
-    CLI Example:
+    CLI Examples:
+
+    .. code-block:: bash
 
         salt-call infoblox.delete_a name=abc.example.com
         salt-call infoblox.delete_a ipv4addr=192.168.3.5
-        salt-call infoblox.delete_a name=acname.example.com allow_array=true
+        salt-call infoblox.delete_a name=acname.example.com allow_array=True
     '''
     r = get_a(name, ipv4addr, allow_array=False, **api_opts)
     if not r:
