@@ -1,11 +1,31 @@
 # -*- coding: utf-8 -*-
 '''
 The service module for macOS
+
 .. versionadded:: 2016.3.0
+
+This module has support for services in the following locations.
+
+.. code-block:: bash
+
+    /System/Library/LaunchDaemons/
+    /System/Library/LaunchAgents/
+    /Library/LaunchDaemons/
+    /Library/LaunchAgents/
+
+    # As of version "2019.2.0" support for user-specific services were added.
+    /Users/foo/Library/LaunchAgents/
+
+.. note::
+    As of the 2019.2.0 release, if a service is located in a ``LaunchAgent``
+    path and a ``runas`` user is NOT specified, the current console user will
+    be used to properly interact with the service.
+
 '''
 from __future__ import absolute_import, unicode_literals, print_function
 
 # Import python libs
+import logging
 import os
 import re
 
@@ -27,6 +47,8 @@ __func_alias__ = {
     'list_': 'list',
 }
 
+log = logging.getLogger(__name__)
+
 
 def __virtual__():
     '''
@@ -51,19 +73,19 @@ def __virtual__():
     return __virtualname__
 
 
-def _get_service(name):
+def _name_in_services(name, services):
     '''
-    Get information about a service.  If the service is not found, raise an
-    error
+    Checks to see if the given service is in the given services.
 
     :param str name: Service label, file name, or full path
 
-    :return: The service information for the service, otherwise an Error
+    :param dict services: The currently available services.
+
+    :return: The service information for the service, otherwise
+    an empty dictionary
+
     :rtype: dict
     '''
-    services = __utils__['mac_utils.available_services']()
-    name = name.lower()
-
     if name in services:
         # Match on label
         return services[name]
@@ -77,8 +99,152 @@ def _get_service(name):
             # Match on basename
             return service
 
-    # Could not find service
-    raise CommandExecutionError('Service not found: {0}'.format(name))
+    return dict()
+
+
+def _get_service(name):
+    '''
+    Get information about a service.  If the service is not found, raise an
+    error
+
+    :param str name: Service label, file name, or full path
+
+    :return: The service information for the service, otherwise an Error
+    :rtype: dict
+    '''
+    services = __utils__['mac_utils.available_services']()
+    name = name.lower()
+
+    service = _name_in_services(name, services)
+
+    # if we would the service we can return it
+    if service:
+        return service
+
+    # if we got here our service is not available, now we can check to see if
+    # we received a cached batch of services, if not we did a fresh check
+    # so we need to raise that the service could not be found.
+    try:
+        if not __context__['using_cached_services']:
+            raise CommandExecutionError('Service not found: {0}'.format(name))
+    except KeyError:
+        pass
+
+    # we used a cached version to check, a service could have been made
+    # between now and then, we should refresh our available services.
+    services = __utils__['mac_utils.available_services'](refresh=True)
+
+    # check to see if we found the service we are looking for.
+    service = _name_in_services(name, services)
+
+    if not service:
+        # Could not find the service after refresh raise.
+        raise CommandExecutionError('Service not found: {0}'.format(name))
+
+    # found it :)
+    return service
+
+
+def _always_running_service(name):
+    '''
+    Check if the service should always be running based on the KeepAlive Key
+    in the service plist.
+
+    :param str name: Service label, file name, or full path
+
+    :return: True if the KeepAlive key is set to True, False if set to False or
+        not set in the plist at all.
+
+    :rtype: bool
+
+    .. versionadded:: 2019.2.0
+    '''
+
+    # get all the info from the launchctl service
+    service_info = show(name)
+
+    # get the value for the KeepAlive key in service plist
+    try:
+        keep_alive = service_info['plist']['KeepAlive']
+    except KeyError:
+        return False
+
+    # check if KeepAlive is True and not just set.
+
+    if isinstance(keep_alive, dict):
+        # check for pathstate
+        for _file, value in six.iteritems(keep_alive.get('PathState', {})):
+            if value is True and os.path.exists(_file):
+                return True
+            elif value is False and not os.path.exists(_file):
+                return True
+
+    if keep_alive is True:
+        return True
+
+    return False
+
+
+def _get_domain_target(name, service_target=False):
+    '''
+    Returns the domain/service target and path for a service. This is used to
+    determine whether or not a service should be loaded in a user space or
+    system space.
+
+    :param str name: Service label, file name, or full path
+
+    :param bool service_target: Whether to return a full
+    service target. This is needed for the enable and disable
+    subcommands of /bin/launchctl. Defaults to False
+
+    :return: Tuple of the domain/service target and the path to the service.
+
+    :rtype: tuple
+
+    .. versionadded:: 2019.2.0
+    '''
+
+    # Get service information
+    service = _get_service(name)
+
+    # get the path to the service
+    path = service['file_path']
+
+    # most of the time we'll be at the system level.
+    domain_target = 'system'
+
+    # check if a LaunchAgent as we should treat these differently.
+    if 'LaunchAgents' in path:
+        # Get the console user so we can service in the correct session
+        uid = __utils__['mac_utils.console_user']()
+        domain_target = 'gui/{}'.format(uid)
+
+    # check to see if we need to make it a full service target.
+    if service_target is True:
+        domain_target = '{}/{}'.format(domain_target, service['plist']['Label'])
+
+    return (domain_target, path)
+
+
+def _launch_agent(name):
+    '''
+    Checks to see if the provided service is a LaunchAgent
+
+    :param str name: Service label, file name, or full path
+
+    :return: True if a LaunchAgent, False if not.
+
+    :rtype: bool
+
+    .. versionadded:: 2019.2.0
+    '''
+
+    # Get the path to the service.
+    path = _get_service(name)['file_path']
+
+    if 'LaunchAgents' not in path:
+        return False
+    return True
 
 
 def show(name):
@@ -153,6 +319,11 @@ def list_(name=None, runas=None):
         service = _get_service(name)
         label = service['plist']['Label']
 
+        # we can assume if we are trying to list a LaunchAgent we need
+        # to run as a user, if not provided, we'll use the console user.
+        if not runas and _launch_agent(name):
+            runas = __utils__['mac_utils.console_user'](username=True)
+
         # Collect information on service: will raise an error if it fails
         return launchctl('list',
                          label,
@@ -182,12 +353,11 @@ def enable(name, runas=None):
 
         salt '*' service.enable org.cups.cupsd
     '''
-    # Get service information and label
-    service = _get_service(name)
-    label = service['plist']['Label']
+    # Get the domain target. enable requires a full <service-target>
+    service_target = _get_domain_target(name, service_target=True)[0]
 
     # Enable the service: will raise an error if it fails
-    return launchctl('enable', 'system/{0}'.format(label), runas=runas)
+    return launchctl('enable', service_target, runas=runas)
 
 
 def disable(name, runas=None):
@@ -208,12 +378,11 @@ def disable(name, runas=None):
 
         salt '*' service.disable org.cups.cupsd
     '''
-    # Get service information and label
-    service = _get_service(name)
-    label = service['plist']['Label']
+    # Get the service target. enable requires a full <service-target>
+    service_target = _get_domain_target(name, service_target=True)[0]
 
     # disable the service: will raise an error if it fails
-    return launchctl('disable', 'system/{0}'.format(label), runas=runas)
+    return launchctl('disable', service_target, runas=runas)
 
 
 def start(name, runas=None):
@@ -237,12 +406,11 @@ def start(name, runas=None):
 
         salt '*' service.start org.cups.cupsd
     '''
-    # Get service information and file path
-    service = _get_service(name)
-    path = service['file_path']
+    # Get the domain target.
+    domain_target, path = _get_domain_target(name)
 
-    # Load the service: will raise an error if it fails
-    return launchctl('load', path, runas=runas)
+    # Load (bootstrap) the service: will raise an error if it fails
+    return launchctl('bootstrap', domain_target, path, runas=runas)
 
 
 def stop(name, runas=None):
@@ -267,12 +435,11 @@ def stop(name, runas=None):
 
         salt '*' service.stop org.cups.cupsd
     '''
-    # Get service information and file path
-    service = _get_service(name)
-    path = service['file_path']
+    # Get the domain target.
+    domain_target, path = _get_domain_target(name)
 
-    # Disable the Launch Daemon: will raise an error if it fails
-    return launchctl('unload', path, runas=runas)
+    # Stop (bootout) the service: will raise an error if it fails
+    return launchctl('bootout', domain_target, path, runas=runas)
 
 
 def restart(name, runas=None):
@@ -313,7 +480,9 @@ def status(name, sig=None, runas=None):
 
     :param str runas: User to run launchctl commands
 
-    :return: The PID for the service if it is running, otherwise an empty string
+    :return: The PID for the service if it is running, or 'loaded' if the
+        service should not always have a PID, or otherwise an empty string
+
     :rtype: str
 
     CLI Example:
@@ -326,6 +495,15 @@ def status(name, sig=None, runas=None):
     if sig:
         return __salt__['status.pid'](sig)
 
+    try:
+        _get_service(name)
+    except CommandExecutionError as msg:
+        log.error(msg)
+        return ''
+
+    if not runas and _launch_agent(name):
+        runas = __utils__['mac_utils.console_user'](username=True)
+
     output = list_(runas=runas)
 
     # Used a string here instead of a list because that's what the linux version
@@ -340,6 +518,12 @@ def status(name, sig=None, runas=None):
                     pids += '\n'
                 pids += line.split()[0]
 
+    # mac services are a little different than other platforms as they may be
+    # set to run on intervals and may not always active with a PID. This will
+    # return a string 'loaded' if it shouldn't always be running and is enabled.
+    if not _always_running_service(name) and enabled(name) and not pids:
+        return 'loaded'
+
     return pids
 
 
@@ -409,7 +593,7 @@ def enabled(name, runas=None):
         return False
 
 
-def disabled(name, runas=None):
+def disabled(name, runas=None, domain='system'):
     '''
     Check if the specified service is not enabled. This is the opposite of
     ``service.enabled``
@@ -418,6 +602,8 @@ def disabled(name, runas=None):
 
     :param str runas: User to run launchctl commands
 
+    :param str domain: domain to check for disabled services. Default is system.
+
     :return: True if the specified service is NOT enabled, otherwise False
     :rtype: bool
 
@@ -427,8 +613,21 @@ def disabled(name, runas=None):
 
         salt '*' service.disabled org.cups.cupsd
     '''
-    # A service is disabled if it is not enabled
-    return not enabled(name, runas=runas)
+
+    disabled = launchctl('print-disabled',
+                         domain,
+                         return_stdout=True,
+                         runas=runas)
+    for service in disabled.split("\n"):
+        if name in service:
+            srv_name = service.split("=>")[0].split("\"")[1]
+            status = service.split("=>")[1]
+            if name != srv_name:
+                pass
+            else:
+                return True if 'true' in status.lower() else False
+
+    return False
 
 
 def get_all(runas=None):
@@ -472,7 +671,6 @@ def get_enabled(runas=None):
     .. code-block:: bash
 
         salt '*' service.get_enabled
-        salt '*' service.get_enabled running=True
     '''
     # Collect list of enabled services
     stdout = list_(runas=runas)
