@@ -18,9 +18,8 @@ except ImportError:  # python 2
 import getpass
 
 # Import salt libs
-import salt.utils.mac_utils
 import salt.utils.platform
-from salt.exceptions import SaltInvocationError
+from salt.exceptions import SaltInvocationError, CommandExecutionError
 
 __virtualname__ = 'system'
 
@@ -45,17 +44,24 @@ def __virtual__():
 
 def _atrun_enabled():
     '''
-    Check to see if atrun is enabled on the system
+    Check to see if atrun is running and enabled on the system
     '''
-    return __salt__['service.enabled']('com.apple.atrun')
+    try:
+        return __salt__['service.list']('com.apple.atrun')
+    except CommandExecutionError:
+        return False
 
 
 def _enable_atrun():
     '''
     Enable and start the atrun daemon
     '''
-    __salt__['service.enable']('com.apple.atrun')
-    __salt__['service.start']('com.apple.atrun')
+    name = 'com.apple.atrun'
+    try:
+        __salt__['service.enable'](name)
+        __salt__['service.start'](name)
+    except CommandExecutionError:
+        return False
     return _atrun_enabled()
 
 
@@ -79,17 +85,18 @@ def halt(at_time=None):
     Halt a running system
 
     :param str at_time: Any valid `at` expression. For example, some valid at
-    expressions could be:
-    - noon
-    - midnight
-    - fri
-    - 9:00 AM
-    - 2:30 PM tomorrow
-    - now + 10 minutes
+        expressions could be:
 
-    Note::
-    If you pass a time only, with no 'AM/PM' designation, you have to double
-    quote the parameter on the command line. For example: '"14:00"'
+        - noon
+        - midnight
+        - fri
+        - 9:00 AM
+        - 2:30 PM tomorrow
+        - now + 10 minutes
+
+    .. note::
+        If you pass a time only, with no 'AM/PM' designation, you have to
+        double quote the parameter on the command line. For example: '"14:00"'
 
     CLI Example:
 
@@ -108,17 +115,18 @@ def sleep(at_time=None):
     sleep.
 
     :param str at_time: Any valid `at` expression. For example, some valid at
-    expressions could be:
-    - noon
-    - midnight
-    - fri
-    - 9:00 AM
-    - 2:30 PM tomorrow
-    - now + 10 minutes
+        expressions could be:
 
-    Note::
-    If you pass a time only, with no 'AM/PM' designation, you have to double
-    quote the parameter on the command line. For example: '"14:00"'
+        - noon
+        - midnight
+        - fri
+        - 9:00 AM
+        - 2:30 PM tomorrow
+        - now + 10 minutes
+
+    .. note::
+        If you pass a time only, with no 'AM/PM' designation, you have to
+        double quote the parameter on the command line. For example: '"14:00"'
 
     CLI Example:
 
@@ -136,17 +144,18 @@ def restart(at_time=None):
     Restart the system
 
     :param str at_time: Any valid `at` expression. For example, some valid at
-    expressions could be:
-    - noon
-    - midnight
-    - fri
-    - 9:00 AM
-    - 2:30 PM tomorrow
-    - now + 10 minutes
+        expressions could be:
 
-    Note::
-    If you pass a time only, with no 'AM/PM' designation, you have to double
-    quote the parameter on the command line. For example: '"14:00"'
+        - noon
+        - midnight
+        - fri
+        - 9:00 AM
+        - 2:30 PM tomorrow
+        - now + 10 minutes
+
+    .. note::
+        If you pass a time only, with no 'AM/PM' designation, you have to
+        double quote the parameter on the command line. For example: '"14:00"'
 
     CLI Example:
 
@@ -164,17 +173,18 @@ def shutdown(at_time=None):
     Shutdown the system
 
     :param str at_time: Any valid `at` expression. For example, some valid at
-    expressions could be:
-    - noon
-    - midnight
-    - fri
-    - 9:00 AM
-    - 2:30 PM tomorrow
-    - now + 10 minutes
+        expressions could be:
 
-    Note::
-    If you pass a time only, with no 'AM/PM' designation, you have to double
-    quote the parameter on the command line. For example: '"14:00"'
+        - noon
+        - midnight
+        - fri
+        - 9:00 AM
+        - 2:30 PM tomorrow
+        - now + 10 minutes
+
+    .. note::
+        If you pass a time only, with no 'AM/PM' designation, you have to
+        double quote the parameter on the command line. For example: '"14:00"'
 
     CLI Example:
 
@@ -199,11 +209,11 @@ def get_remote_login():
 
         salt '*' system.get_remote_login
     '''
-    ret = salt.utils.mac_utils.execute_return_result(
+    ret = __utils__['mac_utils.execute_return_result'](
         'systemsetup -getremotelogin')
 
-    enabled = salt.utils.mac_utils.validate_enabled(
-        salt.utils.mac_utils.parse_return(ret))
+    enabled = __utils__['mac_utils.validate_enabled'](
+        __utils__['mac_utils.parse_return'](ret))
 
     return enabled == 'on'
 
@@ -213,8 +223,8 @@ def set_remote_login(enable):
     Set the remote login (SSH) to either on or off.
 
     :param bool enable: True to enable, False to disable. "On" and "Off" are
-    also acceptable values. Additionally you can pass 1 and 0 to represent True
-    and False respectively
+        also acceptable values. Additionally you can pass 1 and 0 to represent
+        True and False respectively
 
     :return: True if successful, False if not
     :rtype: bool
@@ -225,16 +235,14 @@ def set_remote_login(enable):
 
         salt '*' system.set_remote_login True
     '''
-    state = salt.utils.mac_utils.validate_enabled(enable)
+    state = __utils__['mac_utils.validate_enabled'](enable)
 
     cmd = 'systemsetup -f -setremotelogin {0}'.format(state)
-    salt.utils.mac_utils.execute_return_success(cmd)
+    __utils__['mac_utils.execute_return_success'](cmd)
 
-    return salt.utils.mac_utils.confirm_updated(
-        state,
-        get_remote_login,
-        normalize_ret=True,
-    )
+    return __utils__['mac_utils.confirm_updated'](state,
+                                                  get_remote_login,
+                                                  normalize_ret=True)
 
 
 def get_remote_events():
@@ -250,11 +258,11 @@ def get_remote_events():
 
         salt '*' system.get_remote_events
     '''
-    ret = salt.utils.mac_utils.execute_return_result(
+    ret = __utils__['mac_utils.execute_return_result'](
         'systemsetup -getremoteappleevents')
 
-    enabled = salt.utils.mac_utils.validate_enabled(
-        salt.utils.mac_utils.parse_return(ret))
+    enabled = __utils__['mac_utils.validate_enabled'](
+        __utils__['mac_utils.parse_return'](ret))
 
     return enabled == 'on'
 
@@ -265,8 +273,8 @@ def set_remote_events(enable):
     AppleScripts)
 
     :param bool enable: True to enable, False to disable. "On" and "Off" are
-    also acceptable values. Additionally you can pass 1 and 0 to represent True
-    and False respectively
+        also acceptable values. Additionally you can pass 1 and 0 to represent
+        True and False respectively
 
     :return: True if successful, False if not
     :rtype: bool
@@ -277,12 +285,12 @@ def set_remote_events(enable):
 
         salt '*' system.set_remote_events On
     '''
-    state = salt.utils.mac_utils.validate_enabled(enable)
+    state = __utils__['mac_utils.validate_enabled'](enable)
 
     cmd = 'systemsetup -setremoteappleevents {0}'.format(state)
-    salt.utils.mac_utils.execute_return_success(cmd)
+    __utils__['mac_utils.execute_return_success'](cmd)
 
-    return salt.utils.mac_utils.confirm_updated(
+    return __utils__['mac_utils.confirm_updated'](
         state,
         get_remote_events,
         normalize_ret=True,
@@ -302,10 +310,10 @@ def get_computer_name():
 
         salt '*' system.get_computer_name
     '''
-    ret = salt.utils.mac_utils.execute_return_result(
-        'systemsetup -getcomputername')
+    ret = __utils__['mac_utils.execute_return_result'](
+        'scutil --get ComputerName')
 
-    return salt.utils.mac_utils.parse_return(ret)
+    return __utils__['mac_utils.parse_return'](ret)
 
 
 def set_computer_name(name):
@@ -323,10 +331,10 @@ def set_computer_name(name):
 
         salt '*' system.set_computer_name "Mike's Mac"
     '''
-    cmd = 'systemsetup -setcomputername "{0}"'.format(name)
-    salt.utils.mac_utils.execute_return_success(cmd)
+    cmd = 'scutil --set ComputerName "{0}"'.format(name)
+    __utils__['mac_utils.execute_return_success'](cmd)
 
-    return salt.utils.mac_utils.confirm_updated(
+    return __utils__['mac_utils.confirm_updated'](
         name,
         get_computer_name,
     )
@@ -345,10 +353,10 @@ def get_subnet_name():
 
         salt '*' system.get_subnet_name
     '''
-    ret = salt.utils.mac_utils.execute_return_result(
+    ret = __utils__['mac_utils.execute_return_result'](
         'systemsetup -getlocalsubnetname')
 
-    return salt.utils.mac_utils.parse_return(ret)
+    return __utils__['mac_utils.parse_return'](ret)
 
 
 def set_subnet_name(name):
@@ -371,9 +379,9 @@ def set_subnet_name(name):
         salt '*' system.set_subnet_name "Mike's Mac"
     '''
     cmd = 'systemsetup -setlocalsubnetname "{0}"'.format(name)
-    salt.utils.mac_utils.execute_return_success(cmd)
+    __utils__['mac_utils.execute_return_success'](cmd)
 
-    return salt.utils.mac_utils.confirm_updated(
+    return __utils__['mac_utils.confirm_updated'](
         name,
         get_subnet_name,
     )
@@ -392,10 +400,10 @@ def get_startup_disk():
 
         salt '*' system.get_startup_disk
     '''
-    ret = salt.utils.mac_utils.execute_return_result(
+    ret = __utils__['mac_utils.execute_return_result'](
         'systemsetup -getstartupdisk')
 
-    return salt.utils.mac_utils.parse_return(ret)
+    return __utils__['mac_utils.parse_return'](ret)
 
 
 def list_startup_disks():
@@ -411,7 +419,7 @@ def list_startup_disks():
 
         salt '*' system.list_startup_disks
     '''
-    ret = salt.utils.mac_utils.execute_return_result(
+    ret = __utils__['mac_utils.execute_return_result'](
         'systemsetup -liststartupdisks')
 
     return ret.splitlines()
@@ -441,9 +449,9 @@ def set_startup_disk(path):
         raise SaltInvocationError(msg)
 
     cmd = 'systemsetup -setstartupdisk {0}'.format(path)
-    salt.utils.mac_utils.execute_return_result(cmd)
+    __utils__['mac_utils.execute_return_result'](cmd)
 
-    return salt.utils.mac_utils.confirm_updated(
+    return __utils__['mac_utils.confirm_updated'](
         path,
         get_startup_disk,
     )
@@ -455,7 +463,7 @@ def get_restart_delay():
     power failure.
 
     :return: A string value representing the number of seconds the system will
-    delay restart after power loss
+        delay restart after power loss
     :rtype: str
 
     CLI Example:
@@ -464,10 +472,10 @@ def get_restart_delay():
 
         salt '*' system.get_restart_delay
     '''
-    ret = salt.utils.mac_utils.execute_return_result(
+    ret = __utils__['mac_utils.execute_return_result'](
         'systemsetup -getwaitforstartupafterpowerfailure')
 
-    return salt.utils.mac_utils.parse_return(ret)
+    return __utils__['mac_utils.parse_return'](ret)
 
 
 def set_restart_delay(seconds):
@@ -505,9 +513,9 @@ def set_restart_delay(seconds):
         raise SaltInvocationError(msg)
 
     cmd = 'systemsetup -setwaitforstartupafterpowerfailure {0}'.format(seconds)
-    salt.utils.mac_utils.execute_return_success(cmd)
+    __utils__['mac_utils.execute_return_success'](cmd)
 
-    return salt.utils.mac_utils.confirm_updated(
+    return __utils__['mac_utils.confirm_updated'](
         seconds,
         get_restart_delay,
     )
@@ -527,11 +535,11 @@ def get_disable_keyboard_on_lock():
 
         salt '*' system.get_disable_keyboard_on_lock
     '''
-    ret = salt.utils.mac_utils.execute_return_result(
+    ret = __utils__['mac_utils.execute_return_result'](
         'systemsetup -getdisablekeyboardwhenenclosurelockisengaged')
 
-    enabled = salt.utils.mac_utils.validate_enabled(
-        salt.utils.mac_utils.parse_return(ret))
+    enabled = __utils__['mac_utils.validate_enabled'](
+        __utils__['mac_utils.parse_return'](ret))
 
     return enabled == 'on'
 
@@ -542,8 +550,8 @@ def set_disable_keyboard_on_lock(enable):
     enclosure lock is engaged.
 
     :param bool enable: True to enable, False to disable. "On" and "Off" are
-    also acceptable values. Additionally you can pass 1 and 0 to represent True
-    and False respectively
+        also acceptable values. Additionally you can pass 1 and 0 to represent
+        True and False respectively
 
     :return: True if successful, False if not
     :rtype: bool
@@ -554,13 +562,13 @@ def set_disable_keyboard_on_lock(enable):
 
         salt '*' system.set_disable_keyboard_on_lock False
     '''
-    state = salt.utils.mac_utils.validate_enabled(enable)
+    state = __utils__['mac_utils.validate_enabled'](enable)
 
     cmd = 'systemsetup -setdisablekeyboardwhenenclosurelockisengaged ' \
           '{0}'.format(state)
-    salt.utils.mac_utils.execute_return_success(cmd)
+    __utils__['mac_utils.execute_return_success'](cmd)
 
-    return salt.utils.mac_utils.confirm_updated(
+    return __utils__['mac_utils.confirm_updated'](
         state,
         get_disable_keyboard_on_lock,
         normalize_ret=True,
@@ -580,10 +588,10 @@ def get_boot_arch():
 
         salt '*' system.get_boot_arch
     '''
-    ret = salt.utils.mac_utils.execute_return_result(
+    ret = __utils__['mac_utils.execute_return_result'](
         'systemsetup -getkernelbootarchitecturesetting')
 
-    arch = salt.utils.mac_utils.parse_return(ret)
+    arch = __utils__['mac_utils.parse_return'](ret)
 
     if 'default' in arch:
         return 'default'
@@ -600,20 +608,17 @@ def set_boot_arch(arch='default'):
     Set the kernel to boot in 32 or 64 bit mode on next boot.
 
     .. note::
-
-        This command fails with the following error:
-
-        ``changes to kernel architecture failed to save!``
-
-        The setting is not updated. This is either an apple bug, not available
-        on the test system, or a result of system files now being locked down in
-        macOS (SIP Protection).
+        When this function fails with the error ``changes to kernel
+        architecture failed to save!``, then the boot arch is not updated.
+        This is either an Apple bug, not available on the test system, or a
+        result of system files being locked down in macOS (SIP Protection).
 
     :param str arch: A string representing the desired architecture. If no
-    value is passed, default is assumed. Valid values include:
-    - i386
-    - x86_64
-    - default
+        value is passed, default is assumed. Valid values include:
+
+        - i386
+        - x86_64
+        - default
 
     :return: True if successful, False if not
     :rtype: bool
@@ -631,9 +636,9 @@ def set_boot_arch(arch='default'):
         raise SaltInvocationError(msg)
 
     cmd = 'systemsetup -setkernelbootarchitecture {0}'.format(arch)
-    salt.utils.mac_utils.execute_return_success(cmd)
+    __utils__['mac_utils.execute_return_success'](cmd)
 
-    return salt.utils.mac_utils.confirm_updated(
+    return __utils__['mac_utils.confirm_updated'](
         arch,
         get_boot_arch,
     )
