@@ -10,9 +10,8 @@ import logging
 
 # Import salt libs
 import salt.payload
-import salt.transport
 import salt.utils.args
-import salt.utils.versions
+import salt.transport.client
 from salt.exceptions import SaltReqTimeoutError
 
 # Import 3rd party libs
@@ -84,26 +83,29 @@ def _publish(
             'form': form,
             'id': __opts__['id']}
 
-    channel = salt.transport.Channel.factory(__opts__)
+    channel = salt.transport.client.ReqChannel.factory(__opts__)
     try:
-        peer_data = channel.send(load)
-    except SaltReqTimeoutError:
-        return '\'{0}\' publish timed out'.format(fun)
-    if not peer_data:
-        return {}
-    # CLI args are passed as strings, re-cast to keep time.sleep happy
-    time.sleep(float(timeout))
-    load = {'cmd': 'pub_ret',
-            'id': __opts__['id'],
-            'jid': six.text_type(peer_data['jid'])}
-    ret = channel.send(load)
-    if form == 'clean':
-        cret = {}
-        for host in ret:
-            cret[host] = ret[host]['ret']
-        return cret
-    else:
-        return ret
+        try:
+            peer_data = channel.send(load)
+        except SaltReqTimeoutError:
+            return '\'{0}\' publish timed out'.format(fun)
+        if not peer_data:
+            return {}
+        # CLI args are passed as strings, re-cast to keep time.sleep happy
+        time.sleep(float(timeout))
+        load = {'cmd': 'pub_ret',
+                'id': __opts__['id'],
+                'jid': six.text_type(peer_data['jid'])}
+        ret = channel.send(load)
+        if form == 'clean':
+            cret = {}
+            for host in ret:
+                cret[host] = ret[host]['ret']
+            return cret
+        else:
+            return ret
+    finally:
+        channel.close()
 
 
 def publish(tgt,
@@ -111,8 +113,7 @@ def publish(tgt,
             arg=None,
             tgt_type='glob',
             returner='',
-            timeout=5,
-            expr_form=None):
+            timeout=5):
     '''
     Publish a command from the minion out to other minions.
 
@@ -167,17 +168,6 @@ def publish(tgt,
 
 
     '''
-    # remember to remove the expr_form argument from this function when
-    # performing the cleanup on this deprecation.
-    if expr_form is not None:
-        salt.utils.versions.warn_until(
-            'Fluorine',
-            'the target type should be passed using the \'tgt_type\' '
-            'argument instead of \'expr_form\'. Support for using '
-            '\'expr_form\' will be removed in Salt Fluorine.'
-        )
-        tgt_type = expr_form
-
     return _publish(tgt,
                     fun,
                     arg=arg,
@@ -192,8 +182,7 @@ def full_data(tgt,
               arg=None,
               tgt_type='glob',
               returner='',
-              timeout=5,
-              expr_form=None):
+              timeout=5):
     '''
     Return the full data about the publication, this is invoked in the same
     way as the publish function
@@ -215,17 +204,6 @@ def full_data(tgt,
             salt '*' publish.full_data test.kwarg arg='cheese=spam'
 
     '''
-    # remember to remove the expr_form argument from this function when
-    # performing the cleanup on this deprecation.
-    if expr_form is not None:
-        salt.utils.versions.warn_until(
-            'Fluorine',
-            'the target type should be passed using the \'tgt_type\' '
-            'argument instead of \'expr_form\'. Support for using '
-            '\'expr_form\' will be removed in Salt Fluorine.'
-        )
-        tgt_type = expr_form
-
     return _publish(tgt,
                     fun,
                     arg=arg,
@@ -254,8 +232,10 @@ def runner(fun, arg=None, timeout=5):
             'tmo': timeout,
             'id': __opts__['id']}
 
-    channel = salt.transport.Channel.factory(__opts__)
+    channel = salt.transport.client.ReqChannel.factory(__opts__)
     try:
         return channel.send(load)
     except SaltReqTimeoutError:
         return '\'{0}\' runner publish timed out'.format(fun)
+    finally:
+        channel.close()
