@@ -7,7 +7,7 @@ This is a thin wrapper around Pythons tempfile module
 .. versionadded:: 2015.8.0
 
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 import logging
 import os
