@@ -4,8 +4,9 @@ A salt module for SSL/TLS.
 Can create a Certificate Authority (CA)
 or use Self-Signed certificates.
 
-:depends:   - PyOpenSSL Python module (0.10 or later, 0.14 or later for
-              X509 extension support)
+:depends: PyOpenSSL Python module (0.10 or later, 0.14 or later for X509
+    extension support)
+
 :configuration: Add the following values in /etc/salt/minion for the CA module
     to function properly:
 
@@ -14,7 +15,7 @@ or use Self-Signed certificates.
         ca.cert_base_path: '/etc/pki'
 
 
-CLI Example #1
+CLI Example #1:
 Creating a CA, a server request and its signed certificate:
 
 .. code-block:: bash
@@ -115,6 +116,7 @@ from datetime import datetime
 import salt.utils.data
 import salt.utils.files
 import salt.utils.stringutils
+from salt.exceptions import CommandExecutionError
 from salt.utils.versions import LooseVersion as _LooseVersion
 
 # Import 3rd-party libs
@@ -266,11 +268,8 @@ def _get_basic_info(ca_name, cert, ca_dir=None):
 
     index_file = "{0}/index.txt".format(ca_dir)
 
-    expire_date = _four_digit_year_to_two_digit(
-        datetime.strptime(
-            salt.utils.stringutils.to_str(cert.get_notAfter()),
-            four_digit_year_fmt)
-    )
+    cert = _read_cert(cert)
+    expire_date = _four_digit_year_to_two_digit(_get_expiration_date(cert))
     serial_number = format(cert.get_serial_number(), 'X')
 
     # gotta prepend a /
@@ -548,6 +547,66 @@ def get_ca_signed_key(ca_name,
     return keyp
 
 
+def _read_cert(cert):
+    if isinstance(cert, six.string_types):
+        try:
+            with salt.utils.files.fopen(cert) as rfh:
+                return OpenSSL.crypto.load_certificate(
+                    OpenSSL.crypto.FILETYPE_PEM,
+                    rfh.read()
+                )
+        except Exception:
+            log.exception('Failed to read cert from path %s', cert)
+            return None
+    else:
+        if not hasattr(cert, 'get_notAfter'):
+            log.error('%s is not a valid cert path/object', cert)
+            return None
+        else:
+            return cert
+
+
+def _get_expiration_date(cert):
+    '''
+    Returns a datetime.datetime object
+    '''
+    cert_obj = _read_cert(cert)
+
+    if cert_obj is None:
+        raise CommandExecutionError(
+            'Failed to read cert from {0}, see log for details'.format(cert)
+        )
+
+    return datetime.strptime(
+        salt.utils.stringutils.to_str(cert_obj.get_notAfter()),
+        four_digit_year_fmt
+    )
+
+
+def get_expiration_date(cert, date_format='%Y-%m-%d'):
+    '''
+    .. versionadded:: 2019.2.0
+
+    Get a certificate's expiration date
+
+    cert
+        Full path to the certificate
+
+    date_format
+        By default this will return the expiration date in YYYY-MM-DD format,
+        use this to specify a different strftime format string. Note that the
+        expiration time will be in UTC.
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' tls.get_expiration_date /path/to/foo.crt
+        salt '*' tls.get_expiration_date /path/to/foo.crt date_format='%d/%m/%Y'
+    '''
+    return _get_expiration_date(cert).strftime(date_format)
+
+
 def _check_onlyif_unless(onlyif, unless):
     ret = None
     retcode = __salt__['cmd.retcode']
@@ -1574,12 +1633,15 @@ def create_pkcs12(ca_name, CN, passphrase='', cacert_path=None, replace=False):
     )
 
 
-def cert_info(cert_path, digest='sha256'):
+def cert_info(cert, digest='sha256'):
     '''
     Return information for a particular certificate
 
-    cert_path
-        path to the cert file
+    cert
+        path to the certifiate PEM file or string
+
+        .. versionchanged:: 2018.3.4
+
     digest
         what digest to use for fingerprinting
 
@@ -1588,15 +1650,17 @@ def cert_info(cert_path, digest='sha256'):
     .. code-block:: bash
 
         salt '*' tls.cert_info /dir/for/certs/cert.pem
+
     '''
     # format that OpenSSL returns dates in
     date_fmt = '%Y%m%d%H%M%SZ'
-
-    with salt.utils.files.fopen(cert_path) as cert_file:
-        cert = OpenSSL.crypto.load_certificate(
-            OpenSSL.crypto.FILETYPE_PEM,
-            cert_file.read()
-        )
+    if '-----BEGIN' not in cert:
+        with salt.utils.files.fopen(cert) as cert_file:
+            cert = cert_file.read()
+    cert = OpenSSL.crypto.load_certificate(
+        OpenSSL.crypto.FILETYPE_PEM,
+        cert
+    )
 
     issuer = {}
     for key, value in cert.get_issuer().get_components():
@@ -1635,19 +1699,22 @@ def cert_info(cert_path, digest='sha256'):
         for i in _range(cert.get_extension_count()):
             try:
                 ext = cert.get_extension(i)
-                ret['extensions'][ext.get_short_name()] = str(ext)
+                key = salt.utils.stringutils.to_unicode(ext.get_short_name())
+                ret['extensions'][key] = str(ext).strip()
             except AttributeError:
                 continue
 
     if 'subjectAltName' in ret.get('extensions', {}):
+        valid_entries = ('DNS', 'IP Address')
         valid_names = set()
-        for name in str(ret['extensions']['subjectAltName']).split(", "):
-            if not name.startswith('DNS:'):
+        for name in str(ret['extensions']['subjectAltName']).split(', '):
+            entry, name = name.split(':', 1)
+            if entry not in valid_entries:
                 log.error('Cert {0} has an entry ({1}) which does not start '
-                          'with DNS:'.format(cert_path, name))
+                          'with {2}'.format(ret['subject'], name, '/'.join(valid_entries)))
             else:
-                valid_names.add(name[4:])
-        ret['subject_alt_names'] = ' '.join(valid_names)
+                valid_names.add(name)
+        ret['subject_alt_names'] = list(valid_names)
 
     if hasattr(cert, 'get_signature_algorithm'):
         try:
@@ -1814,14 +1881,8 @@ def revoke_cert(
     except IOError:
         return 'There is no CA named "{0}"'.format(ca_name)
 
-    try:
-        with salt.utils.files.fopen('{}/{}.crt'.format(cert_path,
-                                                 cert_filename)) as rfh:
-            client_cert = OpenSSL.crypto.load_certificate(
-                OpenSSL.crypto.FILETYPE_PEM,
-                rfh.read()
-            )
-    except IOError:
+    client_cert = _read_cert('{0}/{1}.crt'.format(cert_path, cert_filename))
+    if client_cert is None:
         return 'There is no client certificate named "{0}"'.format(CN)
 
     index_file, expire_date, serial_number, subject = _get_basic_info(
