@@ -153,7 +153,7 @@ def lookup_jid(jid,
     try:
         # Check if the return data has an 'out' key. We'll use that as the
         # outputter in the absence of one being passed on the CLI.
-        outputter = data[next(iter(data))].get('out')
+        outputter = returns[next(iter(returns))].get('out')
     except (StopIteration, AttributeError):
         outputter = None
 
@@ -521,8 +521,12 @@ def last_run(ext_source=None,
             log.info('The metadata parameter must be specified as a dictionary')
             return False
 
-    _all_jobs = list_jobs(ext_source, outputter, metadata,
-                          function, target, display_progress)
+    _all_jobs = list_jobs(ext_source=ext_source,
+                          outputter=outputter,
+                          search_metadata=metadata,
+                          search_function=function,
+                          search_target=target,
+                          display_progress=display_progress)
     if _all_jobs:
         last_job = sorted(_all_jobs)[-1]
         return print_job(last_job, ext_source)
