@@ -37,9 +37,9 @@ config:
     myprofile:
         keyid: GKTADJGHEIQSXMKKRBJ08H
         key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
-            region: us-east-1
+        region: us-east-1
 
-.. code-block:: yaml
+.. code-block:: text
 
     Ensure bucket exists:
         boto_s3_bucket.present:
@@ -138,13 +138,15 @@ config:
 
 '''
 
-# Import Python Libs
-from __future__ import absolute_import
+# Import Python libs
+from __future__ import absolute_import, print_function, unicode_literals
+import copy
 import logging
-from copy import deepcopy
-import json
 
 # Import Salt libs
+import salt.utils.json
+
+# Import 3rd-party libs
 from salt.ext import six
 
 log = logging.getLogger(__name__)
@@ -158,7 +160,7 @@ def __virtual__():
 
 
 def _normalize_user(user_dict):
-    ret = deepcopy(user_dict)
+    ret = copy.deepcopy(user_dict)
     # 'Type' is required as input to the AWS API, but not returned as output. So
     # we ignore it everywhere.
     if 'Type' in ret:
@@ -177,7 +179,7 @@ def _prep_acl_for_compare(ACL):
     '''
     Prepares the ACL returned from the AWS API for comparison with a given one.
     '''
-    ret = deepcopy(ACL)
+    ret = copy.deepcopy(ACL)
     ret['Owner'] = _normalize_user(ret['Owner'])
     for item in ret.get('Grants', ()):
         item['Grantee'] = _normalize_user(item.get('Grantee'))
@@ -186,13 +188,13 @@ def _prep_acl_for_compare(ACL):
 
 def _acl_to_grant(ACL, owner_canonical_id):
     if 'AccessControlPolicy' in ACL:
-        ret = deepcopy(ACL['AccessControlPolicy'])
+        ret = copy.deepcopy(ACL['AccessControlPolicy'])
         ret['Owner'] = _normalize_user(ret['Owner'])
         for item in ACL.get('Grants', ()):
             item['Grantee'] = _normalize_user(item.get('Grantee'))
         # If AccessControlPolicy is set, other options are not allowed
         return ret
-    owner_canonical_grant = deepcopy(owner_canonical_id)
+    owner_canonical_grant = copy.deepcopy(owner_canonical_id)
     owner_canonical_grant.update({'Type': 'CanonicalUser'})
     ret = {
         'Grants': [],
@@ -326,7 +328,7 @@ def _compare_replication(current, desired, region, key, keyid, profile):
     Replication accepts a non-ARN role name, but always returns an ARN
     '''
     if desired is not None and desired.get('Role'):
-        desired = deepcopy(desired)
+        desired = copy.deepcopy(desired)
         desired['Role'] = _get_role_arn(desired['Role'],
                                  region=region, key=key, keyid=keyid, profile=profile)
     return __utils__['boto3.json_objs_equal'](current, desired)
@@ -423,7 +425,7 @@ def present(name, Bucket,
         RequestPayment = {'Payer': 'BucketOwner'}
     if Policy:
         if isinstance(Policy, six.string_types):
-            Policy = json.loads(Policy)
+            Policy = salt.utils.json.loads(Policy)
         Policy = __utils__['boto3.ordered'](Policy)
 
     r = __salt__['boto_s3_bucket.exists'](Bucket=Bucket,
@@ -551,7 +553,9 @@ def present(name, Bucket,
                 # Policy description is always returned as a JSON string.
                 # Convert it to JSON now for ease of comparisons later.
                 if isinstance(temp, six.string_types):
-                    current = __utils__['boto3.ordered']({'Policy': json.loads(temp)})
+                    current = __utils__['boto3.ordered'](
+                        {'Policy': salt.utils.json.loads(temp)}
+                    )
         if not comparator(current, desired, region, key, keyid, profile):
             update = True
             if varname == 'ACL':
