@@ -1692,6 +1692,8 @@ def running(name,
         # value here.
         configured_networks = networks
         networks = _parse_networks(networks)
+        if networks:
+            kwargs['networks'] = networks
         image_id = _resolve_image(ret, image, client_timeout)
     except CommandExecutionError as exc:
         ret['result'] = False
