@@ -30,16 +30,17 @@ def policy_present(name, rules):
 
 
     .. code-block:: yaml
-    demo-policy:
-      vault.policy_present:
-        - name: foo/bar
-        - rules: |
-            path "secret/top-secret/*" {
-              policy = "deny"
-            }
-            path "secret/not-very-secret/*" {
-              policy = "write"
-            }
+
+        demo-policy:
+          vault.policy_present:
+            - name: foo/bar
+            - rules: |
+                path "secret/top-secret/*" {
+                  policy = "deny"
+                }
+                path "secret/not-very-secret/*" {
+                  policy = "write"
+                }
 
     '''
     url = "v1/sys/policy/{0}".format(name)
@@ -54,7 +55,7 @@ def policy_present(name, rules):
     except Exception as e:
         return {
             'name': name,
-            'changes': None,
+            'changes': {},
             'result': False,
             'comment': 'Failed to get policy: {0}'.format(e)
         }
@@ -72,10 +73,10 @@ def _create_new_policy(name, rules):
     payload = {'rules': rules}
     url = "v1/sys/policy/{0}".format(name)
     response = __utils__['vault.make_request']('PUT', url, json=payload)
-    if response.status_code != 204:
+    if response.status_code not in [200, 204]:
         return {
             'name': name,
-            'changes': None,
+            'changes': {},
             'result': False,
             'comment': 'Failed to create policy: {0}'.format(response.reason)
         }
@@ -92,7 +93,7 @@ def _handle_existing_policy(name, new_rules, existing_rules):
     ret = {'name': name}
     if new_rules == existing_rules:
         ret['result'] = True
-        ret['changes'] = None
+        ret['changes'] = {}
         ret['comment'] = 'Policy exists, and has the correct content'
         return ret
 
@@ -107,10 +108,10 @@ def _handle_existing_policy(name, new_rules, existing_rules):
 
     url = "v1/sys/policy/{0}".format(name)
     response = __utils__['vault.make_request']('PUT', url, json=payload)
-    if response.status_code != 204:
+    if response.status_code not in [200, 204]:
         return {
             'name': name,
-            'changes': None,
+            'changes': {},
             'result': False,
             'comment': 'Failed to change policy: {0}'.format(response.reason)
         }
