@@ -780,7 +780,7 @@ def pool_running(name,
                                          source_name=(source or {}).get('name', None),
                                          source_format=(source or {}).get('format', None),
                                          transient=transient,
-                                         start=True,
+                                         start=False,
                                          connection=connection,
                                          username=username,
                                          password=password)
@@ -795,11 +795,6 @@ def pool_running(name,
                                         connection=connection,
                                         username=username,
                                         password=password)
-
-            __salt__['virt.pool_start'](name,
-                                        connection=connection,
-                                        username=username,
-                                        password=password)
             ret['changes'][name] = 'Pool defined and started'
             ret['comment'] = 'Pool {0} defined and started'.format(name)
     except libvirt.libvirtError as err:
