@@ -41,7 +41,7 @@ def _handle_sigusr1(sig, stack):
         filename = 'salt-debug-{0}.log'.format(int(time.time()))
         destfile = os.path.join(tempfile.gettempdir(), filename)
         with salt.utils.files.fopen(destfile, 'w') as output:
-            _makepretty(output, salt.utils.stringutils.to_str(stack))
+            _makepretty(output, stack)
 
 
 def _handle_sigusr2(sig, stack):
@@ -99,7 +99,7 @@ def inspect_stack():
     return {'co_name': inspect.stack()[1][3]}
 
 
-def caller_name(skip=2):
+def caller_name(skip=2, include_lineno=False):
     '''
     Get a name of a caller in the format module.class.method
 
@@ -117,6 +117,11 @@ def caller_name(skip=2):
     parentframe = stack[start][0]
 
     name = []
+    if include_lineno is True:
+        try:
+            lineno = inspect.getframeinfo(parentframe).lineno
+        except:  # pylint: disable=bare-except
+            lineno = None
     module = inspect.getmodule(parentframe)
     # `modname` can be None when frame is executed directly in console
     # TODO(techtonik): consider using __main__
@@ -132,4 +137,7 @@ def caller_name(skip=2):
     if codename != '<module>':  # top level usually
         name.append(codename)   # function or a method
     del parentframe
-    return '.'.join(name)
+    fullname = '.'.join(name)
+    if include_lineno and lineno:
+        fullname += ':{}'.format(lineno)
+    return fullname
