@@ -214,7 +214,7 @@ class Schedule(object):
         # dict we treat it like it was there and is True
 
         # Check if we're able to run
-        if not data['run']:
+        if 'run' not in data or not data['run']:
             return data
         if 'jid_include' not in data or data['jid_include']:
             jobcount = 0
@@ -463,7 +463,10 @@ class Schedule(object):
 
         if 'name' not in data:
             data['name'] = name
-        log.info('Running Job: %s', name)
+
+        # Assume run should be True until we check max_running
+        if 'run' not in data:
+            data['run'] = True
 
         if not self.standalone:
             data = self._check_max_running(func,
@@ -472,8 +475,8 @@ class Schedule(object):
                                            datetime.datetime.now())
 
         # Grab run, assume True
-        run = data.get('run', True)
-        if run:
+        if data.get('run'):
+            log.info('Running Job: %s', name)
             self._run_job(func, data)
 
     def enable_schedule(self):
@@ -1593,22 +1596,10 @@ class Schedule(object):
             if '_continue' in data and data['_continue']:
                 run = False
 
-            # If there is no job specific enabled available,
-            # grab the global which defaults to True.
-            if 'enabled' not in data:
-                data['enabled'] = self.enabled
-
-            # If globally disabled, disable the job
-            if not self.enabled:
-                data['enabled'] = self.enabled
-                data['_skip_reason'] = 'disabled'
-                data['_skipped_time'] = now
-                data['_skipped'] = True
-                run = False
-
-            # Job is disabled, set run to False
-            if 'enabled' in data and not data['enabled']:
-                data['_enabled'] = False
+            # If globally disabled or job
+            # is diabled skip the job
+            if not self.enabled or not data.get('enabled', True):
+                log.trace('Job: %s is disabled', job_name)
                 data['_skip_reason'] = 'disabled'
                 data['_skipped_time'] = now
                 data['_skipped'] = True
@@ -1621,14 +1612,6 @@ class Schedule(object):
 
             try:
                 if run:
-                    # Job is disabled, continue
-                    if 'enabled' in data and not data['enabled']:
-                        log.debug('Job: %s is disabled', job_name)
-                        data['_skip_reason'] = 'disabled'
-                        data['_skipped_time'] = now
-                        data['_skipped'] = True
-                        continue
-
                     if 'jid_include' not in data or data['jid_include']:
                         data['jid_include'] = True
                         log.debug('schedule: Job %s was scheduled with jid_include, '
