@@ -10,6 +10,7 @@ import ctypes
 
 # Import Salt Libs
 from salt.exceptions import CommandExecutionError
+from salt.ext.six.moves import range
 
 # Import 3rd Party Libs
 try:
@@ -119,10 +120,16 @@ def get_sid_from_name(name):
     return win32security.ConvertSidToStringSid(sid)
 
 
-def get_current_user():
+def get_current_user(with_domain=True):
     '''
     Gets the user executing the process
 
+    Args:
+
+        with_domain (bool):
+            ``True`` will prepend the user name with the machine name or domain
+            separated by a backslash
+
     Returns:
         str: The user name
     '''
@@ -136,6 +143,8 @@ def get_current_user():
                 user_name = 'SYSTEM'
             elif get_sid_from_name(test_user) == 'S-1-5-18':
                 user_name = 'SYSTEM'
+        elif not with_domain:
+            user_name = win32api.GetUserName()
     except pywintypes.error as exc:
         raise CommandExecutionError(
             'Failed to get current user: {0}'.format(exc))
@@ -173,7 +182,7 @@ def enable_ctrl_logoff_handler():
         )
 
 
-def escape_argument(arg):
+def escape_argument(arg, escape=True):
     '''
     Escape the argument for the cmd.exe shell.
     See http://blogs.msdn.com/b/twistylittlepassagesallalike/archive/2011/04/23/everyone-quotes-arguments-the-wrong-way.aspx
@@ -184,12 +193,19 @@ def escape_argument(arg):
     Args:
         arg (str): a single command line argument to escape for the cmd.exe shell
 
+    Kwargs:
+        escape (bool): True will call the escape_for_cmd_exe() function
+                       which escapes the characters '()%!^"<>&|'. False
+                       will not call the function and only quotes the cmd
+
     Returns:
         str: an escaped string suitable to be passed as a program argument to the cmd.exe shell
     '''
     if not arg or re.search(r'(["\s])', arg):
         arg = '"' + arg.replace('"', r'\"') + '"'
 
+    if not escape:
+        return arg
     return escape_for_cmd_exe(arg)
 
 
@@ -290,3 +306,60 @@ def broadcast_setting_change(message='Environment'):
                                         broadcast_message, SMTO_ABORTIFHUNG,
                                         5000, 0)
     return result == 1
+
+
+def guid_to_squid(guid):
+    '''
+    Converts a GUID   to a compressed guid (SQUID)
+
+    Each Guid has 5 parts separated by '-'. For the first three each one will be
+    totally reversed, and for the remaining two each one will be reversed by
+    every other character. Then the final compressed Guid will be constructed by
+    concatenating all the reversed parts without '-'.
+
+    .. Example::
+
+        Input:                  2BE0FA87-5B36-43CF-95C8-C68D6673FB94
+        Reversed:               78AF0EB2-63B5-FC34-598C-6CD86637BF49
+        Final Compressed Guid:  78AF0EB263B5FC34598C6CD86637BF49
+
+    Args:
+
+        guid (str): A valid GUID
+
+    Returns:
+        str: A valid compressed GUID (SQUID)
+    '''
+    guid_pattern = re.compile(r'^\{(\w{8})-(\w{4})-(\w{4})-(\w\w)(\w\w)-(\w\w)(\w\w)(\w\w)(\w\w)(\w\w)(\w\w)\}$')
+    guid_match = guid_pattern.match(guid)
+    squid = ''
+    if guid_match is not None:
+        for index in range(1, 12):
+            squid += guid_match.group(index)[::-1]
+    return squid
+
+
+def squid_to_guid(squid):
+    '''
+    Converts a compressed GUID (SQUID) back into a GUID
+
+    Args:
+
+        squid (str): A valid compressed GUID
+
+    Returns:
+        str: A valid GUID
+    '''
+    squid_pattern = re.compile(r'^(\w{8})(\w{4})(\w{4})(\w\w)(\w\w)(\w\w)(\w\w)(\w\w)(\w\w)(\w\w)(\w\w)$')
+    squid_match = squid_pattern.match(squid)
+    guid = ''
+    if squid_match is not None:
+        guid = '{' + \
+               squid_match.group(1)[::-1]+'-' + \
+               squid_match.group(2)[::-1]+'-' + \
+               squid_match.group(3)[::-1]+'-' + \
+               squid_match.group(4)[::-1]+squid_match.group(5)[::-1] + '-'
+        for index in range(6, 12):
+            guid += squid_match.group(index)[::-1]
+        guid += '}'
+    return guid
